function [P, u] = householder_matrix(v)
%HOUSEHOLDER_MATRIX Calcola il riflettore di Householder tale che Pv = e1. 
%
% P = HOUSEHOLDER_MATRIX(V) calcola un riflettore di Householder della 
%     forma P = I - b * (u * u') tale che P*v = eye(n, 1); la matrice P è
%     unitaria e simmetrica.
%
% Questa funzione assume che l'input sia un vettore reale. I riflettori di
% Householder si possono utilizzare anche nel caso di vettori complessi, ma
% dei minimi aggiustamenti sono necessari. 
%
% [B, U] = HOUSEHOLDER_MATRIX(V) restituisce uno scalare B ed un vettore U
%     tali che P = eye(n) - B * (U * U'); questo permette di applicare la
%     trasformazione in modo più efficiente, in quanto la matrice P non
%     viene mai calcolata. 

nrm = norm(v);

% Caso speciale: se v è il vettore nullo, l'operazione seguente non è ben
% definita. Ma in questo caso, possiamo semplicemente settare u = 0. 
if nrm == 0
    u = 0;
    b = 0;    
else
    u = v;
    u(1) = u(1) + sign(v(1)) * nrm;
    b = 2 / (u' * u);
end

if nargout == 1
    n = size(v, 1);
    P = eye(n) - b * (u * u');
else
    P = b;
end